<?php

namespace QUI\QueueManager;

use DateTime;
use InvalidArgumentException;
use QUI\QueueManager\Interfaces\QuiqqerQueueWorkerInterface;

use function is_a;
use function is_null;

/**
 * Job to be put in the queue.
 */
class QueueJob
{
    private ?string $workerClass = null;
    public JobStatus $status = JobStatus::QUEUED;

    public readonly JobData $data;
    public ?JobData $result = null;
    public readonly QueueJobAttributes $jobAttributes;
    public readonly JobLog $log;

    /**
     * @param QueueJobUUID|null $uuid (optional) - Is created on INSERT
     * @param QueueJobAttributes|null $jobAttributes (optional) - If omitted, initializes with default attributes
     */
    public function __construct(
        public readonly ?QueueJobUUID $uuid = null,
        ?QueueJobAttributes $jobAttributes = null,
        public ?DateTime $createDate = null,
        public ?DateTime $updateDate = null
    ) {
        $this->data = new JobData();
        $this->log = new JobLog();

        if (is_null($jobAttributes)) {
            $this->jobAttributes = new QueueJobAttributes();
        } else {
            $this->jobAttributes = $jobAttributes;
        }

        if (is_null($this->createDate)) {
            $this->createDate = new DateTime();
        }

        if (is_null($this->updateDate)) {
            $this->updateDate = new DateTime();
        }
    }

    /**
     * Set a class of a QUIQQER worker that executes the job.
     *
     * @param string $workerClass - Must implement QueueWorkerInterface
     * @return void
     */
    public function setQuiqqerQueueWorkerClass(string $workerClass): void
    {
        if (!is_a($workerClass, QuiqqerQueueWorkerInterface::class, true)) {
            throw new InvalidArgumentException(
                "Class $workerClass does not implement " . QuiqqerQueueWorkerInterface::class
            );
        }

        $this->workerClass = $workerClass;
    }

    public function getQuiqqerQueueWorkerClass(): ?string
    {
        return $this->workerClass;
    }

//    /**
//     * Get class of Worker that executes the job
//     *
//     * @return string
//     */
//    public function getWorkerClass()
//    {
//        return $this->workerClass;
//    }

//    /**
//     * Get job data
//     *
//     * @return JobData|null
//     */
//    public function getData(): ?JobData
//    {
//        return $this->data;
//    }

//    /**
//     * Execute the job
//     *
//     * @param bool $closeConnection (optional) - determines if the connection to the server
//     * shall be automatically closed after the job is queued
//     * @return integer - job ID
//     */
//    public function queue($closeConnection = true)
//    {
//        $jobId = QueueManager::addJob($this);
//
//        if ($closeConnection) {
//            QueueManager::closeConnection();
//        }
//
//        return $jobId;
//    }

    /**
     * Get the date this job should be queued at the earliest.
     *
     * @return \DateTime|null
     */
    public function getEarliestQueueDate(): ?\DateTime
    {
        return $this->jobAttributes->earliestQueueDate;
    }
}
